from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.ref import Ref
from ..types import UNSET, Unset

T = TypeVar("T", bound="TextWithRefs")


@attr.s(auto_attribs=True)
class TextWithRefs:
    """ """

    text: str
    refs: Union[Unset, List[Ref]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text = self.text
        refs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.refs, Unset):
            refs = []
            for refs_item_data in self.refs:
                refs_item = refs_item_data.to_dict()

                refs.append(refs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "text": text,
            }
        )
        if refs is not UNSET:
            field_dict["refs"] = refs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text")

        refs = []
        _refs = d.pop("refs", UNSET)
        for refs_item_data in _refs or []:
            refs_item = Ref.from_dict(refs_item_data)

            refs.append(refs_item)

        text_with_refs = cls(
            text=text,
            refs=refs,
        )

        return text_with_refs



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.affiliation import Affiliation
from ..models.pers_name import PersName
from ..types import UNSET, Unset

T = TypeVar("T", bound="Author")


@attr.s(auto_attribs=True)
class Author:
    """ """

    pers_name: PersName
    email: Union[Unset, str] = UNSET
    affiliations: Union[Unset, List[Affiliation]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pers_name = self.pers_name.to_dict()

        email = self.email
        affiliations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.affiliations, Unset):
            affiliations = []
            for affiliations_item_data in self.affiliations:
                affiliations_item = affiliations_item_data.to_dict()

                affiliations.append(affiliations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "persName": pers_name,
            }
        )
        if email is not UNSET:
            field_dict["email"] = email
        if affiliations is not UNSET:
            field_dict["affiliations"] = affiliations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pers_name = PersName.from_dict(d.pop("persName"))

        email = d.pop("email", UNSET)

        affiliations = []
        _affiliations = d.pop("affiliations", UNSET)
        for affiliations_item_data in _affiliations or []:
            affiliations_item = Affiliation.from_dict(affiliations_item_data)

            affiliations.append(affiliations_item)

        author = cls(
            pers_name=pers_name,
            email=email,
            affiliations=affiliations,
        )

        return author



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PersName")


@attr.s(auto_attribs=True)
class PersName:
    """ """

    firstname: Union[Unset, str] = UNSET
    middlename: Union[Unset, str] = UNSET
    surname: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    suffix: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        firstname = self.firstname
        middlename = self.middlename
        surname = self.surname
        title = self.title
        suffix = self.suffix

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if firstname is not UNSET:
            field_dict["firstname"] = firstname
        if middlename is not UNSET:
            field_dict["middlename"] = middlename
        if surname is not UNSET:
            field_dict["surname"] = surname
        if title is not UNSET:
            field_dict["title"] = title
        if suffix is not UNSET:
            field_dict["suffix"] = suffix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        firstname = d.pop("firstname", UNSET)

        middlename = d.pop("middlename", UNSET)

        surname = d.pop("surname", UNSET)

        title = d.pop("title", UNSET)

        suffix = d.pop("suffix", UNSET)

        pers_name = cls(
            firstname=firstname,
            middlename=middlename,
            surname=surname,
            title=title,
            suffix=suffix,
        )

        return pers_name



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Address")


@attr.s(auto_attribs=True)
class Address:
    """ """

    addr_line: Union[Unset, str] = UNSET
    post_code: Union[Unset, str] = UNSET
    settlement: Union[Unset, str] = UNSET
    region: Union[Unset, str] = UNSET
    country: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        addr_line = self.addr_line
        post_code = self.post_code
        settlement = self.settlement
        region = self.region
        country = self.country

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if addr_line is not UNSET:
            field_dict["addrLine"] = addr_line
        if post_code is not UNSET:
            field_dict["postCode"] = post_code
        if settlement is not UNSET:
            field_dict["settlement"] = settlement
        if region is not UNSET:
            field_dict["region"] = region
        if country is not UNSET:
            field_dict["country"] = country

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        addr_line = d.pop("addrLine", UNSET)

        post_code = d.pop("postCode", UNSET)

        settlement = d.pop("settlement", UNSET)

        region = d.pop("region", UNSET)

        country = d.pop("country", UNSET)

        address = cls(
            addr_line=addr_line,
            post_code=post_code,
            settlement=settlement,
            region=region,
            country=country,
        )

        address.additional_properties = d
        return address

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
